#! /usr/bin/env python3
from matplotlib.font_manager import json_dump
from rich import print_json
import typer
from rich.console import Console
from rich.table import Table
import json
import httpx

from snyk_tags import __app_name__, __version__

app = typer.Typer()
console = Console()
'''
List all the different project types and attribute types
'''

# List all project types command
@app.command(help="List all Snyk project types")
def types():
    snykcmd = typer.style("snyk-tags target tag or snyk-tags tag custom", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the attribute types you can apply with {snykcmd}")
    table = Table("Snyk IaC", "Snyk Open Source", "Snyk Container", "Snyk Code")
    table.add_row("terraformconfig", "maven", "dockerfile", "sast")
    table.add_row("dockerfile", "npm", "apk", "")
    table.add_row("terraformplan", "nuget", "deb", "")
    table.add_row("k8sconfig", "gradle", "rpm", "")
    table.add_row("helmconfig", "pip", "linux", "")
    table.add_row("cloudformationconfig", "yarn", "", "")
    table.add_row("armconfig", "gomodules", "", "")
    table.add_row("", "rubygems", "", "")
    table.add_row("", "composer", "", "")
    table.add_row("", "sbt", "", "")
    table.add_row("", "golangdep", "", "")
    table.add_row("", "cocoapods", "", "")
    table.add_row("", "poetry", "", "")
    table.add_row("", "govendor", "", "")
    table.add_row("", "cpp", "", "")
    table.add_row("", "yarn-workspace", "", "")
    table.add_row("", "hex", "", "")
    table.add_row("", "paket", "", "")
    table.add_row("", "golang", "", "")
    console.print(table)

# List Attributes Command
@app.command(help="List all Snyk attribute")
def attributes():
    snykcmd = typer.style("snyk-tags target attributes", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the attribute types you can apply with {snykcmd}")
    table = Table("Criticality", "Environment", "Lifecycle")
    table.add_row("critical", "frontend", "production")
    table.add_row("high", "backend", "development")
    table.add_row("medium", "internal", "sandbox")
    table.add_row("low", "external", "")
    table.add_row("", "mobile", "")
    table.add_row("", "saas", "")
    table.add_row("", "onprem", "")
    table.add_row("", "hosted", "")
    table.add_row("", "distributed", "")
    console.print(table)

# Functions for tags listing command
def create_client(token: str) -> httpx.Client:
        return httpx.Client(
            base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
        )

# Get the tags from a group
def find_tags(
    token: str, group_id: str, jsonflag: bool
) -> tuple:
    with create_client(token=token) as client:
        req = client.get(f"group/{group_id}/tags")
        group = client.get(f"group/{group_id}/orgs").json()
        group_name = group["name"]
        if req.status_code == 200:
            if jsonflag is False:
                print(f"These are the tags in Group: {group_name}")
                table = Table("Key", "Value")
                for tags in req.json().get("tags"):
                    key = tags.get("key")
                    value = tags.get("value")
                    table.add_row(key ,value )
                console.print(table)
            elif jsonflag is True:
                print(json.dumps(req.json()))
        if req.status_code == 404:
            print(f"Group {group_name} not found. Error message: {req.json()}.")
        return req.status_code, req.json()

# List existing tags in a Group Command
@app.command(help="List all existing tags in a Group")
def tags(group_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["GROUP_ID"],
            help="Specify the Group you want to see the tags from"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with Group admin access",
            envvar=["SNYK_TOKEN"]
        ),  json: bool = typer.Option(
            False, "--json", # Default value of comamand
            help=f"Output into json format (default is a table), use --json to change output.",
        )):
    find_tags(snyktkn, group_id, json)



#! /usr/bin/env python3

import logging
import httpx
import typer
from github import Github
from rich import print

from snyk_tags import __app_name__, __version__, attribute

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )

# Apply tags to a specific project
def apply_tag_to_project(
    client: httpx.Client, org_id: str, project_id: str, tag: str, key: str, project_name: str
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }

    req = client.post(f"org/{org_id}/project/{project_id}/tags", data=tag_data)
    
    if req.status_code == 200:
        logging.info(f"Successfully added {tag_data} tags to Project: {project_name}.")
    if req.status_code == 422:
        logging.warning(f"Tag {key}:{tag} is already applied for Project: {project_name}.")
    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}.")
    return req.status_code, req.json()

# Tagging loop
def apply_tags_to_projects(token: str, org_ids: list, name: str, tag: str, key: str) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            isname = 0
            for project in projects.get("projects"):
                if project["name"].startswith(name):
                    apply_tag_to_project(
                            client=client, org_id=org_id, project_id=project["id"], tag=tag, key=key, project_name=project["name"]
                        )
                else:
                    isname=1
            if isname == 1:
                print(f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]")

# GitHub Tagging Loop
def apply_github_owner_to_repo(snyktoken: str, org_ids: list, name: str, githubtoken: str) -> None:
    g = Github(githubtoken)
    with create_client(token=snyktoken) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            isname = 0
            for project in projects.get("projects"):
                if project["name"].startswith(name):
                    repo = g.get_repo(name)
                    apply_tag_to_project(
                            client=client, org_id=org_id, project_id=project["id"], tag=repo.owner.login, key="Owner", project_name=project["name"]
                        )
                else:
                    isname=1
            if isname == 1:
                print(f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]")

# Coloured variables for output
crit = typer.style("critical, high, medium, low", bold=True, fg=typer.colors.MAGENTA)
enviro = typer.style("frontend, backend, internal, 	external, mobile, saas, onprem, hosted, distributed", bold=True, fg=typer.colors.MAGENTA)
life = typer.style("production, development, sandbox", bold=True, fg=typer.colors.MAGENTA)
repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)

@app.command(help=f"Apply a custom tag to a target, for example {repoexample}")
def tag(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify the Organization ID where you want to apply the tag"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with org admin access",
            envvar=["SNYK_TOKEN"]
        ),  target: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the target, for example {repoexample}"
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):
    typer.secho(f"\nAdding the tag key {tagKey} and tag value {tagValue} to projects within {target} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
    apply_tags_to_projects(snyktkn,[org_id], target, tagValue, tagKey)

# GitHub Code Owner Tagging
@app.command(help=f"Add the GitHub code owner as a tag to the specified repo in Snyk, for example {repoexample}")
def github(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify the Organization ID where you want to apply the tag"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with org admin access",
            envvar=["SNYK_TOKEN"]
        ),  target: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the repo, for example {repoexample}"
        ),  githubtkn: str = typer.Option(
            ..., # Default value of comamand
            help="GitHub Personal Access Token with access to the repository",
            envvar=["GITHUB_TOKEN"]
        )
    ):
    typer.secho(f"\nAdding the Owner tag to projects within {target} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
    apply_github_owner_to_repo(snyktkn,[org_id], target, githubtkn)

# Collection command to apply the attributes to the collection
@app.command(help=f"Apply attributes to a target, for example {repoexample}")
def attributes(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify the Organization ID where you want to apply the attributes"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with org admin access",
            envvar=["SNYK_TOKEN"]
        ),  target: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the project collection, for example {repoexample}"
        ),  criticality: str = typer.Option(
            ..., # Default value of comamand
            help=f"Criticality attribute: {crit}"
        ),  environment: str = typer.Option(
            ..., # Default value of comamand
            help=f"Environment attribute: {enviro}"
        ),  lifecycle: str = typer.Option(
            ..., # Default value of comamand
            help=f"Lifecycle attribute: {life}"
        )
    ):
    typer.secho(f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
    attribute.apply_attributes_to_projects(snyktkn,[org_id], target, [criticality], [environment], [lifecycle])


#! /usr/bin/env python3

import logging
import httpx
import typer
import json
from rich import print

from snyk_tags import __app_name__, __version__

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}", 'Content-Type': 'application/json'} 
    )

# Apply attributes to a specific project
def apply_attributes_to_project(
    client: httpx.Client, org_id: str, project_id: str, criticality: list, environment: list, lifecycle: list, project_name: str
) -> tuple:
    attribute_data = {
        "criticality": criticality,
        "environment": environment,
        "lifecycle": lifecycle
    }
    
    req = client.post(f"org/{org_id}/project/{project_id}/attributes", data=json.dumps(attribute_data))
    
    attribute_data = typer.style(attribute_data, bold=True, fg=typer.colors.MAGENTA)
    criticality = typer.style(criticality, bold=True, fg=typer.colors.MAGENTA)
    environment = typer.style(environment, bold=True, fg=typer.colors.MAGENTA)
    lifecycle = typer.style(lifecycle, bold=True, fg=typer.colors.MAGENTA)
    if req.status_code == 200:
        logging.info(f"Successfully added {criticality},{environment},{lifecycle} attributes to Project: {project_name}.")
    if req.status_code == 422:
        logging.warning(f"Data {attribute_data} cannot be processed, make sure you have written the correct values (refer to help or Readme) and that they are in low caps. Error message: {req.json()}.")
    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project: {project_name}. Error message: {req.json()}.")
    if req.status_code == 500:
        logging.error(f"Error message: {req.json()}. Please contact eric.fernandez@snyk.io.")
    return req.status_code, req.json()

# Apply attributes to projects within a collection
def apply_attributes_to_projects(token: str, org_ids: list, name: str, criticality: list, environment: list, lifecycle: list) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            isname = 0
            for project in projects.get("projects"):
                if project["name"].startswith(name):
                    apply_attributes_to_project(
                            client=client, org_id=org_id, project_id=project["id"], criticality=criticality, environment=environment, lifecycle=lifecycle, project_name=project["name"]
                        )
                else:
                    isname=1
            if isname == 1:
                print(f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]")



#! /usr/bin/env python3
import typer
import httpx
from rich import print
from snyk import SnykClient

app = typer.Typer()

# Remove tags from a specific project
def remove_tag_from_project(
    token: str, org_id: str, project_id: str, tag: str, key: str, project_name: str
) -> tuple:
    
    client = SnykClient(token=token)
    try:
        client.organizations.get(org_id).projects.get(project_id).tags.delete(key, tag)
        print(f"Removing tag {key}:{tag} from {project_name}")
    except Exception as e:
        if "422" in str(e):
            print(f"The tag {key}:{tag} has already been removed from Project: {project_name}")
        elif "404" in str(e):
            print((f"Project not found. Project: {project_name}. Error message: {e}."))
        else:
            print(f"Unknown error {e}")
    
# Remove tag loop with pysnyk 
def remove_tags_from_projects(token: str, org_id: list, name: str, tag: str, key: str) -> None:
    client = SnykClient(token=token)
    projects = client.organizations.get(org_id).projects.all()
    isname = 0
    for project in projects:
        if project.name.startswith(name):
            remove_tag_from_project(
                    token=token, org_id=org_id, project_id=project.id, tag=tag, key=key, project_name=project.name
                )
        else:
            isname=1
    if isname == 1:
        print(f"[bold red]{name}[/bold red] is not a valid target, please check it is a target within the organization e.g. [bold blue]snyk-labs/snyk-goof[/bold blue]")

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )

# Apply tags to a specific project
def remove_tag_from_group(
    token: str, group_id: str, force: bool, tag: str, key: str 
) -> tuple:

    if force is True:
        tag_data = {
            "key": key,
            "value": tag,
            "force" : force 
        }
    else: 
        tag_data = {
            "key": key,
            "value": tag
        }

    with create_client(token=token) as client:
        req = client.post(f"group/{group_id}/tags/delete", data=tag_data)
        group = client.get(f"group/{group_id}/orgs").json()
        group_name = group["name"]
    
        if req.status_code == 200:
            print(f"Successfully removed {key}:{tag} from Group: {group_name}")
        if req.status_code == 403:
            print(f"The tag {key}:{tag} has entities attached in Group: {group_name}  Error message: {req.json()}")
        if req.status_code == 422:
            print(f"The tag {key}:{tag} has already been removed from Group: {group_name}  Error message: {req.json()}")
        if req.status_code == 404:
            print(f"Tag {key}:{tag} not found in {group_name}. Error message: {req.json()}.")
        return req.status_code, req.json()

repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)

@app.command(help=f"Remove a tag from a target, for example {repoexample}")
def tag_from_target(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify the Organization ID to remove the tag from"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with org admin access",
            envvar=["SNYK_TOKEN"]
        ),  target: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the target, for example {repoexample}"
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):
    typer.secho(f"\nRemoving {tagKey}:{tagValue} from projects within {target}", bold=True)
    remove_tags_from_projects(snyktkn,org_id, target, tagValue, tagKey)

@app.command(help=f"Remove a tag from a Group, this can be forced through --force")
def tag_from_group(group_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["GROUP_ID"],
            help="Specify the Group where you want to remove the tag from"
        ),  snyktkn: str = typer.Option(
            ..., # Default value of comamand
            help="Snyk API token with Group admin access",
            envvar=["SNYK_TOKEN"]
        ),  force: bool = typer.Option(
            False, "--force", # Default value of comamand
            help=f"Force delete tag that has entities (default is false), use --force to turn into True.",
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):
    typer.secho(f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}", bold=True)
    remove_tag_from_group(snyktkn,group_id, force, tagValue, tagKey)


#! /usr/bin/env python3
import typer
from pathlib import Path
from typing import List
import csv
from snyk_tags import collection, attribute, remove
from rich import print
import json

app = typer.Typer()
repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)
tagexample = typer.style("org-id,target,key,value", bold=True, fg=typer.colors.MAGENTA)
attributesexample = typer.style("org-id,target,criticality,environment,lifecycle", bold=True, fg=typer.colors.MAGENTA)
removetaggroupexample = typer.style("key,value", bold=True, fg=typer.colors.MAGENTA)
removetagtargetexample = typer.style("org-id,target,key,value", bold=True, fg=typer.colors.MAGENTA)

@app.command(help=f"Apply a custom tag from a .csv or .json to a target, for example {repoexample} \n\n The .csv or .json must be in the format {tagexample}")
def target_tag(file: List[Path] = typer.Option(...,help=f".csv or .json file with the format {tagexample}"),
    snyktkn: str = typer.Option(...,help="Snyk API token with org admin access",envvar=["SNYK_TOKEN"])):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(f"\nAdding the tag key {key} and tag value {value} to projects within {target} for easy filtering via the UI", bold=True)
                    collection.apply_tags_to_projects(snyktkn, [org_id], target, value, key)
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(f"\nAdding the tag key {key} and tag value {value} to projects within {target} for easy filtering via the UI", bold=True)
                    collection.apply_tags_to_projects(snyktkn, [org_id], target, value, key)
                openfile.close()
            else:
                print(f"The file {openfile.name} is not valid, it must be either a .csv or a .json")
                openfile.close()
        else:
            print(f"The file or path does not exist")

@app.command(help=f"Apply attributes from a .csv or .json to a target, for example {repoexample} \n\n The .csv or .json must be in the format {attributesexample}")
def target_attributes(file: List[Path] = typer.Option(...,help=f".csv or .json file with the format {attributesexample}"),
    snyktkn: str = typer.Option(...,help="Snyk API token with org admin access",envvar=["SNYK_TOKEN"])):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    criticality = row.get("criticality")
                    environment = row.get("environment")
                    lifecycle = row.get("lifecycle")
                    typer.secho(f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
                    attribute.apply_attributes_to_projects(snyktkn, [org_id], target, [criticality], [environment], [lifecycle])
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    criticality = row.get("criticality")
                    environment = row.get("environment")
                    lifecycle = row.get("lifecycle")
                    typer.secho(f"\nAdding the attributes {criticality}, {environment} and {lifecycle} to projects within {target} for easy filtering via the UI", bold=True, fg=typer.colors.MAGENTA)
                    attribute.apply_attributes_to_projects(snyktkn, [org_id], target, [criticality], [environment], [lifecycle])
                openfile.close()
            else:
                print(f"The file {openfile.name} is not valid, it must be either a .csv or a .json")
                openfile.close()
        else:
            print(f"The file or path does not exist")

@app.command(help=f"Remove tags from a Group with .csv or .json, this can be forced through --force")
def remove_tag_from_group(file: List[Path] = typer.Option(...,help=f".csv or .json file with the format {removetaggroupexample}"),
    group_id: str = typer.Option(...,envvar=["GROUP_ID"],help="Specify the Group where you want to remove the tag from"),
    snyktkn: str = typer.Option(...,help="Snyk API token with Group admin access",envvar=["SNYK_TOKEN"]),
    force: bool = typer.Option(False, "--force",help=f"Force delete tag that has entities (default is false), use --force to turn into True.",)):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    tagKey = row.get("key")
                    tagValue = row.get("value")
                    typer.secho(f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}", bold=True)
                    remove.remove_tag_from_group(snyktkn, group_id, force, tagValue, tagKey)
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    tagKey = row.get("key")
                    tagValue = row.get("value")
                    typer.secho(f"\nRemoving {tagKey}:{tagValue} from Group ID: {group_id}", bold=True)
                    remove.remove_tag_from_group(snyktkn, group_id, force, tagValue, tagKey)
                openfile.close()
            else:
                print(f"The file {openfile.name} is not valid, it must be either a .csv or a .json")
                openfile.close()
        else:
            print(f"The file or path does not exist")

@app.command(help=f"Remove a tag from a target with .csv or .json, for example {repoexample}")
def remove_tag_from_target(file: List[Path] = typer.Option(...,help=f".csv or .json file with the format {removetagtargetexample}"),
    snyktkn: str = typer.Option(...,  help="Snyk API token with org admin access",envvar=["SNYK_TOKEN"])):
    for path in file:
        if path.is_file():
            openfile = open(path)
            if ".csv" in openfile.name:
                csvreader = csv.DictReader(openfile)
                for row in csvreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(f"\nRemoving {key}:{value} from projects within {target}", bold=True)
                    remove.remove_tags_from_projects(snyktkn,org_id, target, value, key)
                openfile.close()
            elif ".json" in openfile.name:
                jsonreader = json.load(openfile)
                for row in jsonreader:
                    org_id = row.get("org-id")
                    target = row.get("target")
                    key = row.get("key")
                    value = row.get("value")
                    typer.secho(f"\nRemoving {key}:{value} from projects within {target}", bold=True)
                    remove.remove_tags_from_projects(snyktkn,org_id, target, value, key)
                openfile.close()
            else:
                print(f"The file {openfile.name} is not valid, it must be either a .csv or a .json")
                openfile.close()
        else:
            print(f"The file or path does not exist")